package com.qmth.teachcloud.common.api.api;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.teachcloud.common.entity.SysRole;
import com.qmth.teachcloud.common.service.SysRoleService;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 角色表 前端控制器
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Api(tags = "角色Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.sys}/role")
public class SysRoleController {

    @Autowired
    private SysRoleService sysRoleService;

    /**
     * 查询
     *
     * @param name
     * @param enable
     * @param pageNumber
     * @param pageSize
     * @return
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public Result list(@RequestParam(value = "name", required = false) String name,
                       @RequestParam(value = "enable", required = false) Boolean enable,
                       @RequestParam(value = "pageNumber", required = true) Integer pageNumber,
                       @RequestParam(value = "pageSize", required = true) Integer pageSize) {
        IPage<SysRole> userDtos = sysRoleService.list(name, enable, pageNumber, pageSize);
        return ResultUtil.ok(userDtos);
    }

    /**
     * 新增用户时查询角色方法
     * @return
     */
    @ApiOperation(value = "新增用户时查询角色方法")
    @RequestMapping(value = "/list_to_user", method = RequestMethod.POST)
    public Result listToUser() {
        List<SysRole> userDtos = sysRoleService.listToUser();
        return ResultUtil.ok(userDtos);
    }

    /**
     * 新增/修改
     *
     * @param role
     * @return
     */
    @ApiOperation(value = "新增/修改")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public Result save(@RequestBody SysRole role) {
        boolean isSuccess = sysRoleService.saveRole(role);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 启用/禁用
     *
     * @param role
     * @return
     */
    @ApiOperation(value = "启用/禁用")
    @RequestMapping(value = "/enable", method = RequestMethod.POST)
    public Result enable(@RequestBody SysRole role) {
        boolean isSuccess = sysRoleService.enable(role);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "删除")
    @RequestMapping(value = "/remove", method = RequestMethod.POST)
    public Result remove(Long id) {
        boolean isSuccess = sysRoleService.remove(id);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 用户已绑定角色列表
     *
     * @param userId
     * @return
     */
    @ApiOperation(value = "用户已绑定角色列表")
    @RequestMapping(value = "/get_user_roles", method = RequestMethod.POST)
    public Result getUserRoles(@RequestParam(value = "userId", required = true) Long userId) {
        List<Long> list = sysRoleService.getUserRoles(userId);
        return ResultUtil.ok(list);
    }
}

