package com.qmth.teachcloud.common.service;

import com.qmth.teachcloud.common.bean.auth.AuthBean;
import com.qmth.teachcloud.common.entity.*;
import com.qmth.teachcloud.common.enums.PrivilegePropertyEnum;

import java.util.List;

/**
 * @Description: 缓存操作service
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/25
 */
public interface CacheService {

    /**
     * 添加用户缓存
     *
     * @param userId
     * @return
     */
    public SysUser userCache(Long userId);

    /**
     * 修改用户缓存
     *
     * @param userId
     * @return
     */
    public SysUser updateUserCache(Long userId);

    /**
     * 删除用户缓存
     *
     * @param userId
     */
    public void removeUserCache(Long userId);

    /**
     * 添加用户鉴权缓存
     *
     * @param userId
     * @return
     */
    public AuthBean userAuthCache(Long userId);

    /**
     * 修改用户鉴权缓存
     *
     * @param userId
     * @return
     */
    public AuthBean updateUserAuthCache(Long userId);

    /**
     * 删除用户鉴权缓存
     *
     * @param userId
     */
    public void removeUserAuthCache(Long userId);

    /**
     * 添加学校缓存
     *
     * @param schoolId
     * @return
     */
    public BasicSchool schoolCache(Long schoolId);

    /**
     * 添加学校缓存
     *
     * @param code
     * @return
     */
    public BasicSchool schoolCache(String code);

    /**
     * 添加机构缓存
     *
     * @param orgId
     * @return
     */
    public SysOrg orgCache(Long orgId);

    /**
     * 获取鉴权url
     *
     * @param privilegePropertyEnum
     * @return
     */
    public List<String> privilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum);

    /**
     * 修改鉴权url
     *
     * @param privilegePropertyEnum
     * @return
     */
    public List<String> updatePrivilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum);


    /**
     * 删除鉴权url
     *
     * @param privilegePropertyEnum
     */
    public void removePrivilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum);

    /**
     * 添加角色权限缓存
     *
     * @param roleId
     * @return
     */
    public List<SysRolePrivilege> rolePrivilegeCache(Long roleId);

    /**
     * 修改角色权限缓存
     *
     * @param roleId
     * @return
     */
    public List<SysRolePrivilege> updateRolePrivilegeCache(Long roleId);

    /**
     * 删除角色权限缓存
     *
     * @param roleId
     */
    public void removeRolePrivilegeCache(Long roleId);

    /**
     * 添加用户角色权限缓存
     *
     * @param userId
     * @return
     */
    public List<SysUserRole> userRolePrivilegeCache(Long userId);

    /**
     * 修改用户角色权限缓存
     *
     * @param userId
     * @return
     */
    public List<SysUserRole> updateUserRolePrivilegeCache(Long userId);

    /**
     * 删除用户角色权限缓存
     *
     * @param userId
     */
    public void removeUserRolePrivilegeCache(Long userId);
}
