package com.qmth.teachcloud.common.service;

import com.qmth.teachcloud.common.bean.auth.AuthBean;
import com.qmth.teachcloud.common.bean.dto.PrivilegeCacheDto;
import com.qmth.teachcloud.common.bean.result.LoginResult;
import com.qmth.teachcloud.common.bean.result.RolePrivilegeResult;
import com.qmth.teachcloud.common.entity.BasicAttachment;
import com.qmth.teachcloud.common.entity.SysRolePrivilege;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.entity.SysUserRole;
import com.qmth.teachcloud.common.enums.PrivilegePropertyEnum;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @Description: 公共服务service
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/26
 */
public interface TeachcloudCommonService {

    /**
     * 新增用户权限
     *
     * @param sysUser
     * @param roleId
     */
    public void addUserRolePrivilege(SysUser sysUser, Long roleId);

    /**
     * 获取用户角色
     *
     * @param userId
     * @return
     */
    public List<SysUserRole> getUserRolePrivilege(Long userId);

    /**
     * 获取角色权限
     *
     * @param roleId
     * @return
     */
    public List<SysRolePrivilege> getRolePrivilege(Long roleId);

    /**
     * 获取用户权限
     *
     * @param userId
     * @return
     */
    public AuthBean getUserAuth(Long userId);

    /**
     * 获取鉴权url
     *
     * @param privilegePropertyEnum
     * @return
     */
    public List<String> getPrivilegeUrl(PrivilegePropertyEnum privilegePropertyEnum);

    /**
     * 删除用户信息
     *
     * @param userId
     */
    public void removeUserInfo(Long userId) throws NoSuchAlgorithmException;

    /**
     * 修改角色状态
     *
     * @param roleIds
     * @param enable
     */
    public void updateRoleEnable(Long[] roleIds, Boolean enable);

    /**
     * 查询用户或角色权限信息
     *
     * @param privilegeCacheDtoList
     * @return
     */
    public RolePrivilegeResult getUserOrRolePrivilege(List<PrivilegeCacheDto> privilegeCacheDtoList);

    /**
     * 文件预览
     *
     * @param path
     * @return
     */
    public String filePreview(String path);

    /**
     * 文件预览
     *
     * @param path
     * @return
     */
    public String filePreviewByPathAndType(String path, String type, Boolean isExpire);

    /**
     * 文件预览
     *
     * @param attachmentId
     * @return
     */
    public Map<String, String> filePreviewByAttachmentId(Long attachmentId, Boolean isExpire);

    /**
     * 题卡html文件上传，读取文件内容
     *
     * @param path
     * @return
     */
    public String readFileContent(String path);

    /**
     * 获取文件流
     *
     * @param path
     * @param pdf
     * @return
     */
    public InputStream getFileInputStream(String path, boolean pdf) throws IOException;

    /**
     * 获取文件byte数据
     *
     * @param path
     * @param pdf
     * @return
     */
    public byte[] getFileByte(String path, boolean pdf) throws IOException;

    /**
     * 获取文件
     *
     * @param path
     * @param pdf
     * @return
     */
    public File getFile(String path, boolean pdf) throws IOException;

    /**
     * 下载文件到本地
     *
     * @param rootPath   保存目录
     * @param attachment 附件表对象
     * @return
     */
    public File copyFile(String rootPath, String fileName, BasicAttachment attachment);

    /**
     * 下载文件到本地并压缩返回
     *
     * @param filePath 文件根目录
     * @param files    需要下载的文件集合
     */
    public void downloadFileAndZip(HttpServletResponse response, String filePath, List<File> files);

    /**
     * 下载文件到本地并压缩返回
     *
     * @param filePath 文件根目录
     * @param time     zip文件名（时间戳）
     */
    public void downloadFileAndZip(HttpServletResponse response, String filePath, long time);

    /**
     * 根据orgId查询所有子机构ID集合
     *
     * @return
     */
    public Set<Long> listSubOrgIds(Long orgId);

    /**
     * 登录公用
     *
     * @param password
     * @param sysUser
     * @return
     */
    public LoginResult login(String password, SysUser sysUser) throws NoSuchAlgorithmException;
}
