package com.qmth.teachcloud.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.teachcloud.common.bean.auth.AuthBean;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.*;
import com.qmth.teachcloud.common.enums.PrivilegePropertyEnum;
import com.qmth.teachcloud.common.service.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.CachePut;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @Description: 缓存操作serviceImpl 实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/25
 */
@Service(value = "applicationCacheService")
public class CacheServiceImpl implements CacheService {
    private final static Logger log = LoggerFactory.getLogger(CacheServiceImpl.class);

    @Resource
    SysUserService sysUserService;

    @Resource
    BasicSchoolService basicSchoolService;

    @Resource
    TeachcloudCommonService commonService;

    @Resource
    SysOrgService sysOrgService;

    /**
     * 添加用户缓存
     *
     * @param userId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.USER_ACCOUNT_CACHE, key = "#p0")
    public SysUser userCache(Long userId) {
        return sysUserService.getById(userId);
    }

    /**
     * 修改用户缓存
     *
     * @param userId
     * @return
     */
    @Override
    @CachePut(value = SystemConstant.USER_ACCOUNT_CACHE, key = "#p0")
    public SysUser updateUserCache(Long userId) {
        return sysUserService.getById(userId);
    }

    /**
     * 删除用户缓存
     *
     * @param userId
     */
    @Override
    @CacheEvict(value = SystemConstant.USER_ACCOUNT_CACHE, key = "#p0")
    public void removeUserCache(Long userId) {

    }

    /**
     * 添加用户鉴权缓存
     *
     * @param userId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.USER_OAUTH_CACHE, key = "#p0")
    public AuthBean userAuthCache(Long userId) {
        return commonService.getUserAuth(userId);
    }

    /**
     * 修改用户鉴权缓存
     *
     * @param userId
     * @return
     */
    @Override
    @CachePut(value = SystemConstant.USER_OAUTH_CACHE, key = "#p0")
    public AuthBean updateUserAuthCache(Long userId) {
        return commonService.getUserAuth(userId);
    }

    /**
     * 删除用户鉴权缓存
     *
     * @param userId
     */
    @Override
    @CacheEvict(value = SystemConstant.USER_OAUTH_CACHE, key = "#p0")
    public void removeUserAuthCache(Long userId) {

    }

    /**
     * 添加学校缓存
     *
     * @param schoolId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.SCHOOL_CACHE, key = "#p0")
    public BasicSchool schoolCache(Long schoolId) {
        return basicSchoolService.getById(schoolId);
    }

    /**
     * 添加学校缓存
     *
     * @param code
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.SCHOOL_CODE_CACHE, key = "#p0")
    public BasicSchool schoolCache(String code) {
        QueryWrapper<BasicSchool> basicSchoolQueryWrapper = new QueryWrapper<>();
        basicSchoolQueryWrapper.lambda().eq(BasicSchool::getCode, code);
        return basicSchoolService.getOne(basicSchoolQueryWrapper);
    }

    /**
     * 添加机构缓存
     *
     * @param orgId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.ORG_CACHE, key = "#p0")
    public SysOrg orgCache(Long orgId) {
        return sysOrgService.getById(orgId);
    }

    /**
     * 获取鉴权url
     *
     * @param privilegePropertyEnum
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.PRIVILEGE_URL_CACHE, key = "#p0")
    public List<String> privilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum) {
        return commonService.getPrivilegeUrl(privilegePropertyEnum);
    }

    /**
     * 修改鉴权url
     *
     * @param privilegePropertyEnum
     * @return
     */
    @Override
    @CachePut(value = SystemConstant.PRIVILEGE_URL_CACHE, key = "#p0")
    public List<String> updatePrivilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum) {
        return commonService.getPrivilegeUrl(privilegePropertyEnum);
    }

    /**
     * 删除鉴权url
     *
     * @param privilegePropertyEnum
     */
    @Override
    @CacheEvict(value = SystemConstant.PRIVILEGE_URL_CACHE, key = "#p0")
    public void removePrivilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum) {

    }

    /**
     * 获取角色权限缓存
     *
     * @param roleId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.ROLE_PRIVILEGE_CACHE, key = "#p0")
    public List<SysRolePrivilege> rolePrivilegeCache(Long roleId) {
        return commonService.getRolePrivilege(roleId);
    }

    /**
     * 修改角色权限缓存
     *
     * @param roleId
     * @return
     */
    @Override
    @CachePut(value = SystemConstant.ROLE_PRIVILEGE_CACHE, key = "#p0")
    public List<SysRolePrivilege> updateRolePrivilegeCache(Long roleId) {
        return commonService.getRolePrivilege(roleId);
    }

    /**
     * 删除角色权限缓存
     *
     * @param roleId
     */
    @Override
    @CacheEvict(value = SystemConstant.ROLE_PRIVILEGE_CACHE, key = "#p0")
    public void removeRolePrivilegeCache(Long roleId) {

    }

    /**
     * 添加用户角色权限缓存
     *
     * @param userId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.USER_ROLE_PRIVILEGE_CACHE, key = "#p0")
    public List<SysUserRole> userRolePrivilegeCache(Long userId) {
        return commonService.getUserRolePrivilege(userId);
    }

    /**
     * 修改用户角色权限缓存
     *
     * @param userId
     * @return
     */
    @Override
    @CachePut(value = SystemConstant.USER_ROLE_PRIVILEGE_CACHE, key = "#p0")
    public List<SysUserRole> updateUserRolePrivilegeCache(Long userId) {
        return commonService.getUserRolePrivilege(userId);
    }

    /**
     * 删除用户角色权限缓存
     *
     * @param userId
     */
    @Override
    @CacheEvict(value = SystemConstant.USER_ROLE_PRIVILEGE_CACHE, key = "#p0")
    public void removeUserRolePrivilegeCache(Long userId) {

    }
}
