package com.qmth.teachcloud.report.business.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.util.ExcelUtil;
import com.qmth.teachcloud.common.util.ServletUtil;
import com.qmth.teachcloud.common.util.excel.ExcelError;
import com.qmth.teachcloud.report.business.bean.dto.ExamStudentTypeDto;
import com.qmth.teachcloud.report.business.bean.result.InspectCollegeResult;
import com.qmth.teachcloud.report.business.bean.result.TBExamStudentResult;
import com.qmth.teachcloud.report.business.bean.result.TBStudentReportResult;
import com.qmth.teachcloud.report.business.bean.result.TeachCollegeResult;
import com.qmth.teachcloud.report.business.entity.TBExamStudent;
import com.qmth.teachcloud.report.business.mapper.TBExamStudentMapper;
import com.qmth.teachcloud.report.business.service.TBExamStudentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 考生表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2021-06-01
 */
@Service
public class TBExamStudentServiceImpl extends ServiceImpl<TBExamStudentMapper, TBExamStudent> implements TBExamStudentService {

    @Resource
    TBExamStudentMapper tbExamStudentMapper;

    @Autowired
    BasicCourseService basicCourseService;

    /**
     * 查询开课学院列表
     *
     * @param schoolId
     * @param examId
     * @param courseCode
     * @return
     */
    @Override
    public List<TBExamStudentResult> selectTeachCollegeList(Long schoolId, Long examId, String courseCode) {
        return tbExamStudentMapper.selectTeachCollegeList(schoolId, examId, courseCode);
    }

    /**
     * 查询考察学院列表
     *
     * @param schoolId
     * @param examId
     * @param courseCode
     * @return
     */
    @Override
    public List<TBExamStudentResult> selectInspectCollegeList(Long schoolId, Long examId, String courseCode) {
        return tbExamStudentMapper.selectInspectCollegeList(schoolId, examId, courseCode);
    }

    /**
     * 查询老师列表
     *
     * @param schoolId
     * @param examId
     * @param courseCode
     * @return
     */
    @Override
    public List<TBExamStudentResult> selectTeachList(Long schoolId, Long examId, String courseCode) {
        return tbExamStudentMapper.selectTeachList(schoolId, examId, courseCode);
    }

    /**
     * 查询班级列表
     *
     * @param schoolId
     * @param examId
     * @param courseCode
     * @return
     */
    @Override
    public List<TBExamStudentResult> selectClassList(Long schoolId, Long examId, String courseCode) {
        return tbExamStudentMapper.selectClassList(schoolId, examId, courseCode);
    }

    @Override
    public IPage<TeachCollegeResult> listTeachCollegeResult(String semester, Long examId, String courseCode, Boolean current, Long inspectCollegeId, Long teacherId, String studentParam, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Page<TeachCollegeResult> page = new Page<>(pageNumber, pageSize);
        IPage<TeachCollegeResult> listPage = tbExamStudentMapper.listTeachCollegeResult(page, schoolId, semester, examId, courseCode, current, inspectCollegeId, teacherId, studentParam);
        return listPage;
    }

    @Override
    public void importFile(Long examId, MultipartFile file) throws IOException, NoSuchFieldException {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        List<LinkedMultiValueMap<Integer, Object>> finalList = ExcelUtil.excelReader(file.getInputStream(), Lists.newArrayList(ExamStudentTypeDto.class), (finalExcelList, finalColumnNameList, finalExcelErrorList) -> {
            List<ExcelError> excelErrorTemp = new ArrayList<>();
            // 只允许导入一个sheet
            if (finalExcelList.size() > 1) {
                throw ExceptionResultEnum.ERROR.exception("excel中只允许有一个sheet");
            }
            for (int i = 0; i < finalExcelList.size(); i++) {
                LinkedMultiValueMap<Integer, Object> excelMap = finalExcelList.get(i);
                List<Object> examTaskTempList = excelMap.get(i);
                for (int y = 0; y < examTaskTempList.size(); y++) {
                    ExamStudentTypeDto userImportDto = (ExamStudentTypeDto) examTaskTempList.get(y);
                    if (StringUtils.isBlank(userImportDto.getTicketNumber())) {
                        excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[准考证号]必填"));
                    }
                    if (StringUtils.isBlank(userImportDto.getCourseCode())) {
                        excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[课程代码]必填"));
                    }
                    if (StringUtils.isBlank(userImportDto.getCourseName())) {
                        excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[课程名称]必填"));
                    }

                    // 课程是否存在并且与名称对应
                    QueryWrapper<BasicCourse> queryWrapper = new QueryWrapper<>();
                    queryWrapper.lambda().eq(BasicCourse::getSchoolId, schoolId).eq(BasicCourse::getCode, userImportDto.getCourseCode());
                    BasicCourse course = basicCourseService.getOne(queryWrapper);
                    if (course == null) {
                        excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[课程代码]在系统中不存在"));
                    } else if (!course.getName().equals(userImportDto.getCourseName())) {
                        excelErrorTemp.add(new ExcelError(y + 1, "excel第" + (i + 1) + "个sheet第" + (y + 1) + "行[课程名称]与系统中不匹配"));
                    }
                }
            }
            if (excelErrorTemp.size() > 0) {
                List<String> errors = excelErrorTemp.stream().map(m -> m.getExcelErrorType()).collect(Collectors.toList());
                throw ExceptionResultEnum.ERROR.exception(JSONObject.toJSONString(errors));
            }
            return finalExcelList;
        });

        for (int i = 0; i < finalList.size(); i++) {
            LinkedMultiValueMap<Integer, Object> excelMap = finalList.get(i);
            List<Object> tempList = excelMap.get(i);
            for (int y = 0; y < tempList.size(); y++) {
                ExamStudentTypeDto examStudentTypeDto = (ExamStudentTypeDto) tempList.get(y);
                if (StringUtils.isNotBlank(examStudentTypeDto.getCurrent())) {
                    UpdateWrapper<TBExamStudent> updateWrapper = new UpdateWrapper<>();
                    updateWrapper.lambda().set(TBExamStudent::getCurrent, "应届".equals(examStudentTypeDto.getCurrent().trim()))
                            .eq(TBExamStudent::getExamId, examId)
                            .eq(TBExamStudent::getTicketNumber, examStudentTypeDto.getTicketNumber().trim())
                            .eq(TBExamStudent::getCourseCode, examStudentTypeDto.getCourseCode().trim());
                    this.update(updateWrapper);
                }
            }
        }
    }

    @Override
    public void exportTeachCollegeResult(String semester, Long examId, String courseCode, Boolean current, Long inspectCollegeId, Long teacherId, String studentParam, HttpServletResponse response) throws Exception {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        List<TeachCollegeResult> list = tbExamStudentMapper.listTeachCollegeResult(schoolId, semester, examId, courseCode, current, inspectCollegeId, teacherId, studentParam);
        ExcelUtil.excelExport("开课成绩查询", TeachCollegeResult.class, list, response);
    }

    @Override
    public IPage<InspectCollegeResult> listInspectCollegeResult(String semester, Long examId, String courseCode, Boolean current, Long teachCollegeId, Long teacherId, String studentParam, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Page<InspectCollegeResult> page = new Page<>(pageNumber, pageSize);
        IPage<InspectCollegeResult> listPage = tbExamStudentMapper.listInspectCollegeResult(page, schoolId, semester, examId, courseCode, current, teachCollegeId, teacherId, studentParam);
        return listPage;
    }

    @Override
    public void exportInspectCollegeResult(String semester, Long examId, String courseCode, Boolean current, Long teachCollegeId, Long teacherId, String studentParam, HttpServletResponse response) throws Exception {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        List<InspectCollegeResult> list = tbExamStudentMapper.listInspectCollegeResult(schoolId, semester, examId, courseCode, current, teachCollegeId, teacherId, studentParam);
        ExcelUtil.excelExport("开课成绩查询", TeachCollegeResult.class, list, response);
    }

    /**
     * 学生报告预览-列表接口
     *
     * @param iPage
     * @param schoolId
     * @param examId
     * @param collegeId
     * @param courseCode
     * @param clazzId
     * @param absent
     * @return
     */
    @Override
    public IPage<TBStudentReportResult> reportList(IPage<Map> iPage, Long schoolId, Long examId, Long collegeId, String courseCode, Long clazzId, Boolean absent) {
        return tbExamStudentMapper.reportList(iPage, schoolId, examId, collegeId, courseCode, clazzId, absent);
    }
}
