package com.qmth.teachcloud.report.api;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.util.Result;
import com.qmth.teachcloud.common.util.ResultUtil;
import com.qmth.teachcloud.report.business.bean.result.TBExamResult;
import com.qmth.teachcloud.report.business.bean.result.TBExamStudentResult;
import com.qmth.teachcloud.report.business.entity.TBExam;
import com.qmth.teachcloud.report.business.entity.TBExamCourse;
import com.qmth.teachcloud.report.business.enums.SemesterEnum;
import com.qmth.teachcloud.report.business.service.TBExamCourseService;
import com.qmth.teachcloud.report.business.service.TBExamService;
import com.qmth.teachcloud.report.business.service.TBExamStudentService;
import io.swagger.annotations.*;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Api(tags = "教研分析学校公用Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.reportSchool}")
public class SysReportController {

    @Resource
    TBExamService tbExamService;

    @Resource
    TBExamCourseService tbExamCourseService;

    @Resource
    TBExamStudentService tbExamStudentService;

    @ApiOperation(value = "学期列表")
    @RequestMapping(value = "/common/list_semester", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "考试信息", response = TBExamResult.class)})
    public Result listSemester(@ApiParam(value = "学校id", required = true) @RequestParam String schoolId) {
        QueryWrapper<TBExam> tbExamQueryWrapper = new QueryWrapper<>();
        tbExamQueryWrapper.select(" DISTINCT id,semester ").eq("school_id", SystemConstant.convertIdToLong(schoolId));
        List<TBExam> tbExamList = tbExamService.list(tbExamQueryWrapper);
        List<TBExamResult> tbExamResultList = null;
        if (Objects.nonNull(tbExamList) && tbExamList.size() > 0) {
            tbExamResultList = new ArrayList<>();
            for (TBExam tbExam : tbExamList) {
                tbExamResultList.add(new TBExamResult(tbExam.getId(), tbExam.getSemester()));
            }
        }
        return ResultUtil.ok(tbExamResultList);
    }

    @ApiOperation(value = "考试列表")
    @RequestMapping(value = "/common/list_exam", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "考试信息", response = TBExamResult.class)})
    public Result listExam(@ApiParam(value = "学校id", required = true) @RequestParam String schoolId,
                           @ApiParam(value = "学期枚举", required = true) @RequestParam SemesterEnum semester) {
        QueryWrapper<TBExam> tbExamQueryWrapper = new QueryWrapper<>();
        tbExamQueryWrapper.select(" DISTINCT id,exam_name as examName,exam_code as examCode ")
                .eq("school_id", SystemConstant.convertIdToLong(schoolId))
                .eq("semester", semester);
        List<TBExam> tbExamList = tbExamService.list(tbExamQueryWrapper);
        List<TBExamResult> tbExamResultList = null;
        if (Objects.nonNull(tbExamList) && tbExamList.size() > 0) {
            tbExamResultList = new ArrayList<>();
            for (TBExam tbExam : tbExamList) {
                tbExamResultList.add(new TBExamResult(tbExam.getId(), tbExam.getExamName(), tbExam.getExamCode()));
            }
        }
        return ResultUtil.ok(tbExamResultList);
    }

    @ApiOperation(value = "课程列表")
    @RequestMapping(value = "/common/list_course", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "科目信息", response = TBExamCourse.class)})
    public Result listCourse(@ApiParam(value = "学校id", required = true) @RequestParam String schoolId,
                             @ApiParam(value = "考试id", required = true) @RequestParam String examId) {
        QueryWrapper<TBExamCourse> tbExamCourseQueryWrapper = new QueryWrapper<>();
        tbExamCourseQueryWrapper.lambda().eq(TBExamCourse::getSchoolId, SystemConstant.convertIdToLong(schoolId));
        tbExamCourseQueryWrapper.lambda().eq(TBExamCourse::getExamId, SystemConstant.convertIdToLong(examId));
        return ResultUtil.ok(tbExamCourseService.list(tbExamCourseQueryWrapper));
    }

    @ApiOperation(value = "开课学院列表")
    @RequestMapping(value = "/common/list_teach_college", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "考生信息", response = TBExamStudentResult.class)})
    public Result listTeachCollege(@ApiParam(value = "学校id", required = true) @RequestParam String schoolId,
                                   @ApiParam(value = "考试id", required = true) @RequestParam String examId,
                                   @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode) {
        return ResultUtil.ok(tbExamStudentService.selectTeachCollegeList(SystemConstant.convertIdToLong(schoolId), SystemConstant.convertIdToLong(examId), courseCode));
    }

    @ApiOperation(value = "考察学院列表")
    @RequestMapping(value = "/common/list_inspect_college", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "考生信息", response = TBExamStudentResult.class)})
    public Result listInspectCollege(@ApiParam(value = "学校id", required = true) @RequestParam String schoolId,
                                     @ApiParam(value = "考试id", required = true) @RequestParam String examId,
                                     @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode) {
        return ResultUtil.ok(tbExamStudentService.selectInspectCollegeList(SystemConstant.convertIdToLong(schoolId), SystemConstant.convertIdToLong(examId), courseCode));
    }

    @ApiOperation(value = "教师列表")
    @RequestMapping(value = "/common/list_teacher", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "考生信息", response = TBExamStudentResult.class)})
    public Result listTeacher(@ApiParam(value = "学校id", required = true) @RequestParam String schoolId,
                              @ApiParam(value = "考试id", required = true) @RequestParam String examId,
                              @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode) {
        return ResultUtil.ok(tbExamStudentService.selectTeachList(SystemConstant.convertIdToLong(schoolId), SystemConstant.convertIdToLong(examId), courseCode));
    }

    @ApiOperation(value = "班级列表")
    @RequestMapping(value = "/common/list_class", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "考生信息", response = TBExamStudentResult.class)})
    public Result listClass(@ApiParam(value = "学校id", required = true) @RequestParam String schoolId,
                            @ApiParam(value = "考试id", required = true) @RequestParam String examId,
                            @ApiParam(value = "科目编码", required = true) @RequestParam String courseCode) {
        return ResultUtil.ok(tbExamStudentService.selectClassList(SystemConstant.convertIdToLong(schoolId), SystemConstant.convertIdToLong(examId), courseCode));
    }
}
